#!/usr/bin/env python

"""
XSL-based form templating.

Copyright (C) 2005 Paul Boddie <paul@boddie.org.uk>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
"""

import Constants
import libxsltmod, libxml2mod
import libxml2dom

"""
import libxml2

def quiet(context, s):
    pass

libxml2.registerErrorHandler(quiet, None)
"""

def path_to_node(node, attribute_ref, name, multivalue=0):

    """
    Generate an XSLForms path to the given 'node', producing an attribute
    reference if 'attribute_ref' is true; for example:

    /package$1/discriminators$5/discriminator$1/category

    Otherwise an element reference is produced; for example:

    /package$1/discriminators$5/discriminator$1

    Use the given 'name' to complete the path if an attribute reference is
    required (and if a genuine attribute is found at the context node -
    otherwise 'name' will be None and the context node will be treated like an
    attribute).

    If 'multivalue' is true, produce an attribute reference using the given
    'name' of the following form:

    /package$1/categories$1/category$$name
    """

    l = []
    # Skip attribute reference.
    if node.nodeType == node.ATTRIBUTE_NODE:
        node = node.parentNode
    # Manually insert the attribute name if defined.
    if attribute_ref:
        # A real attribute is referenced.
        if name is not None:
            l.insert(0, name)
            if multivalue:
                l.insert(0, Constants.multi_separator)
                l.insert(0, node.nodeName)
                node = node.parentNode
            l.insert(0, Constants.path_separator)
        # Otherwise, treat the element name as an attribute name.
        else:
            l.insert(0, node.nodeName)
            l.insert(0, Constants.path_separator)
            node = node.parentNode
    # Element references.
    while node is not None and node.nodeType != node.DOCUMENT_NODE:
        l.insert(0, str(int(node.xpath("count(preceding-sibling::*) + 1"))))
        l.insert(0, Constants.pair_separator)
        l.insert(0, node.nodeName)
        l.insert(0, Constants.path_separator)
        node = node.parentNode
    return "".join(l)

def path_to_context(context, attribute_ref, multivalue_name=None):

    """
    As a libxslt extension function, return a string containing the XSLForms
    path to the 'context' node, using the special "this-name" variable to
    complete the path if an attribute reference is required (as indicated by
    'attribute_ref' being set to true). If 'multivalue_name' is set, produce a
    reference to a multivalued field using the given string as the attribute
    name.
    """

    context = libxml2mod.xmlXPathParserGetContext(context)
    transform_context = libxsltmod.xsltXPathGetTransformContext(context)
    name_var = libxsltmod.xsltVariableLookup(transform_context, "this-name", None)
    if multivalue_name is not None:
        name = multivalue_name
        multivalue = 1
    elif name_var is not None:
        name = libxml2mod.xmlNodeGetContent(name_var[0])
        multivalue = 0
    else:
        name = None
        multivalue = 0
    node = libxml2dom.Node(libxml2mod.xmlXPathGetContextNode(context))
    return path_to_node(node, attribute_ref, name, multivalue)

def this_position(context):
    #print "this_position"
    r = path_to_context(context, 0)
    return r.encode("utf-8")

def field_name(context):
    #print "field_name"
    r = path_to_context(context, 1)
    return r.encode("utf-8")

def multi_field_name(context, multivalue_name):
    #print "multi_field_name"
    r = path_to_context(context, 1, multivalue_name)
    return r.encode("utf-8")

def new_field(context, name):
    #print "new_field"
    r = path_to_context(context, 0) + "/" + name
    return r.encode("utf-8")

def other_field_names(context, nodes):
    #print "other_field_names"
    names = []
    for node in nodes:
        name = path_to_node(libxml2dom.Node(node), 0, None, 0)
        if name not in names:
            names.append(name)
    r = ",".join(names)
    return r.encode("utf-8")

def other_multi_field_names(context, multivalue_name, nodes):
    #print "other_multi_field_names"
    names = []
    for node in nodes:
        name = path_to_node(libxml2dom.Node(node), 1, multivalue_name, 1)
        if name not in names:
            names.append(name)
    r = ",".join(names)
    return r.encode("utf-8")

libxsltmod.xsltRegisterExtModuleFunction("this-position", "http://www.boddie.org.uk/ns/xmltools/template", this_position)
libxsltmod.xsltRegisterExtModuleFunction("field-name", "http://www.boddie.org.uk/ns/xmltools/template", field_name)
libxsltmod.xsltRegisterExtModuleFunction("multi-field-name", "http://www.boddie.org.uk/ns/xmltools/template", multi_field_name)
libxsltmod.xsltRegisterExtModuleFunction("new-field", "http://www.boddie.org.uk/ns/xmltools/template", new_field)
libxsltmod.xsltRegisterExtModuleFunction("other-field-names", "http://www.boddie.org.uk/ns/xmltools/template", other_field_names)
libxsltmod.xsltRegisterExtModuleFunction("other-multi-field-names", "http://www.boddie.org.uk/ns/xmltools/template", other_multi_field_names)

def get_field_name(field_or_multi_name):
    return field_or_multi_name.split(Constants.multi_separator)[0]

def get_element_path(field_or_multi_name):

    """
    Convert the given 'field_or_multi_name' back to an XPath reference.
    For example:
    /configuration$1/details$1/base-system$$value -> /*[position() = 1]/*[position() = 1]/base-system
    """

    field_name = get_field_name(field_or_multi_name)
    parts = field_name.split(Constants.path_separator)
    new_parts = []
    for part in parts:
        path_parts = part.split(Constants.pair_separator)
        if len(path_parts) == 2:
            new_parts.append("*[position() = " + path_parts[1] + "]")
        else:
            new_parts.append(path_parts[0])
    return "/".join(new_parts)

# vim: tabstop=4 expandtab shiftwidth=4
